@@ -9,8 +9,10 @@ from typing import Union, List
 
 # noinspection PyPackageRequirements minIO_requirement
 import certifi
-import minio.definitions
+import minio
+import minio.datatypes
 import minio.error
+import minio.helpers
 # noinspection PyPackageRequirements minIO_requirement
 import urllib3
 from django.core.files import File
@@ -18,7 +20,6 @@ from django.core.files.storage import Storage
 from django.core.files.uploadedfile import InMemoryUploadedFile
 from django.utils.deconstruct import deconstructible
 from django.utils.timezone import utc
-from minio import Minio
 
 from .utils import MinioServerStatus, PrivatePublicMixedError, ConfigurationError, get_setting
 
@@ -56,11 +57,18 @@ class MinioBackend(Storage):
 
         self._REPLACE_EXISTING = kwargs.get('replace_existing', False)
 
-        self.__CLIENT: Union[Minio, None] = None
+        self.__CLIENT: Union[minio.Minio, None] = None
+        self.__CLIENT_EXTERNAL: Union[minio.Minio, None] = None
         self.__MINIO_ENDPOINT: str = get_setting("MINIO_ENDPOINT")
+        self.__MINIO_EXTERNAL_ENDPOINT: str = get_setting("MINIO_EXTERNAL_ENDPOINT", self.__MINIO_ENDPOINT)
         self.__MINIO_ACCESS_KEY: str = get_setting("MINIO_ACCESS_KEY")
         self.__MINIO_SECRET_KEY: str = get_setting("MINIO_SECRET_KEY")
         self.__MINIO_USE_HTTPS: bool = get_setting("MINIO_USE_HTTPS")
+        self.__MINIO_EXTERNAL_ENDPOINT_USE_HTTPS: bool = get_setting("MINIO_EXTERNAL_ENDPOINT_USE_HTTPS", self.__MINIO_USE_HTTPS)
+
+        self.__BASE_URL = ("https://" if self.__MINIO_USE_HTTPS else "http://") + self.__MINIO_ENDPOINT
+        self.__BASE_URL_EXTERNAL = ("https://" if self.__MINIO_EXTERNAL_ENDPOINT_USE_HTTPS else "http://") + self.__MINIO_EXTERNAL_ENDPOINT
+        self.__SAME_ENDPOINTS = self.__MINIO_ENDPOINT == self.__MINIO_EXTERNAL_ENDPOINT
 
         self.PRIVATE_BUCKETS: List[str] = get_setting("MINIO_PRIVATE_BUCKETS", [])
         self.PUBLIC_BUCKETS: List[str] = get_setting("MINIO_PUBLIC_BUCKETS", [])
@@ -106,6 +114,15 @@ class MinioBackend(Storage):
         )
         return file_path.as_posix()
 
+    def get_available_name(self, name, max_length=None):
+        """
+        Return a filename that's free on the target storage system and
+        available for new content to be written to.
+        """
+        if self._REPLACE_EXISTING:
+            return name
+        return super(MinioBackend, self).get_available_name(name, max_length)
+
     def _open(self, object_name, mode='rb', **kwargs):
         """
         Implements the Storage._open(name,mode='rb') method
@@ -124,12 +141,12 @@ class MinioBackend(Storage):
             resp.release_conn()
         return file
 
-    def stat(self, name: str) -> Union[minio.definitions.Object, bool]:
+    def stat(self, name: str) -> Union[minio.datatypes.Object, bool]:
         object_name = Path(name).as_posix()
         try:
             obj = self.client.stat_object(self._BUCKET_NAME, object_name=object_name)
             return obj
-        except (minio.error.NoSuchKey, minio.error.NoSuchBucket):
+        except (minio.error.S3Error, minio.error.ServerError):
             return False
         except urllib3.exceptions.MaxRetryError:
             return False
@@ -152,7 +169,7 @@ class MinioBackend(Storage):
         return False
 
     def listdir(self, bucket_name: str):
-        objects = self.client.list_objects_v2(bucket_name=bucket_name, recursive=True)
+        objects = self.client.list_objects(bucket_name=bucket_name, recursive=True)
         return [(obj.object_name, obj) for obj in objects]
 
     def size(self, name: str) -> int:
@@ -171,15 +188,22 @@ class MinioBackend(Storage):
         :return: (str) URL to object
         """
         if self.is_bucket_public:
-            # noinspection PyProtectedMember
-            return f'{self.client._endpoint_url}/{self._BUCKET_NAME}/{name}'
+            return f'{self.base_url_external}/{self._BUCKET_NAME}/{name}'
 
         try:
-            return self.client.presigned_get_object(
-                bucket_name=self._BUCKET_NAME,
-                object_name=name.encode('utf-8'),
-                expires=get_setting("MINIO_URL_EXPIRY_HOURS", timedelta(days=7))  # Default is 7 days
-            )
+            if self.same_endpoints:
+                u: str = self.client.presigned_get_object(
+                    bucket_name=self._BUCKET_NAME,
+                    object_name=name.encode('utf-8'),
+                    expires=get_setting("MINIO_URL_EXPIRY_HOURS", timedelta(days=7))  # Default is 7 days
+                )
+            else:
+                u: str = self.client_external.presigned_get_object(
+                    bucket_name=self._BUCKET_NAME,
+                    object_name=name.encode('utf-8'),
+                    expires=get_setting("MINIO_URL_EXPIRY_HOURS", timedelta(days=7))  # Default is 7 days
+                )
+            return u
         except urllib3.exceptions.MaxRetryError:
             raise ConnectionError("Couldn't connect to Minio. Check django_minio_backend parameters in Django-Settings")
 
@@ -221,6 +245,13 @@ class MinioBackend(Storage):
         MinioBackend
     """
 
+    @property
+    def same_endpoints(self) -> bool:
+        """
+        Returns True if (self.__MINIO_ENDPOINT == self.__MINIO_EXTERNAL_ENDPOINT)
+        """
+        return self.__SAME_ENDPOINTS
+
     @property
     def bucket(self) -> str:
         return self._BUCKET_NAME
@@ -237,12 +268,11 @@ class MinioBackend(Storage):
 
         with urllib3.PoolManager(cert_reqs=ssl.CERT_REQUIRED, ca_certs=certifi.where()) as http:
             try:
-                scheme = "https" if self.__MINIO_USE_HTTPS else "http"
-                r = http.request('GET', f'{scheme}://{self.__MINIO_ENDPOINT}/minio/index.html')
+                r = http.request('GET', f'{self.__BASE_URL}/minio/index.html')
                 return MinioServerStatus(r)
             except urllib3.exceptions.MaxRetryError as e:
                 mss = MinioServerStatus(None)
-                mss.add_message(f'Could not open connection to {self.__MINIO_ENDPOINT}/minio/index.html\n'
+                mss.add_message(f'Could not open connection to {self.__BASE_URL}/minio/index.html\n'
                                 f'Reason: {e}')
                 return mss
             except Exception as e:
@@ -251,16 +281,30 @@ class MinioBackend(Storage):
                 return mss
 
     @property
-    def client(self) -> Minio:
+    def client(self) -> minio.Minio:
         if not self.__CLIENT:
             self.new_client()
             return self.__CLIENT
         return self.__CLIENT
 
-    def new_client(self):
+    @property
+    def client_external(self) -> minio.Minio:
+        if not self.__CLIENT_EXTERNAL:
+            self.new_client(internal=False)
+            return self.__CLIENT_EXTERNAL
+        return self.__CLIENT_EXTERNAL
+
+    @property
+    def base_url(self) -> str:
+        return self.__BASE_URL
+
+    @property
+    def base_url_external(self) -> str:
+        return self.__BASE_URL_EXTERNAL
+
+    def new_client(self, internal: bool = True):
         """
-        Instantiates a new Minio client and
-        :return:
+        Instantiates a new Minio client and assigns it to their respective class variable
         """
         # Safety Guards
         if not self.PRIVATE_BUCKETS or not self.PUBLIC_BUCKETS:
@@ -270,14 +314,17 @@ class MinioBackend(Storage):
                 'is not configured properly in your settings.py (or equivalent)'
             )
 
-        mc = Minio(
-            endpoint=self.__MINIO_ENDPOINT,
+        mc = minio.Minio(
+            endpoint=self.__MINIO_ENDPOINT if internal else self.__MINIO_EXTERNAL_ENDPOINT,
             access_key=self.__MINIO_ACCESS_KEY,
             secret_key=self.__MINIO_SECRET_KEY,
-            secure=self.__MINIO_USE_HTTPS,
+            secure=self.__MINIO_USE_HTTPS if internal else self.__MINIO_EXTERNAL_ENDPOINT_USE_HTTPS,
             http_client=self.HTTP_CLIENT,
         )
-        self.__CLIENT = mc
+        if internal:
+            self.__CLIENT = mc
+        else:
+            self.__CLIENT_EXTERNAL = mc
 
     # MAINTENANCE
     def check_bucket_existence(self):
